<?php
/**
 * DBCart 商城系统
 *
 * ==========================================================================
 * @link      https://www.dbcart.cn/
 * @copyright 北京珑大钜商科技有限公司，并保留所有权利。
 * @license   https://www.dbcart.cn/license.html License
 * ==========================================================================
 *
 * @author    静静的风 <baron@loongdom.cn>
 *
 */

namespace Admin\Plugin;

use Admin\Data\Common;
use Admin\Data\Config;
use Admin\Entity\AdminUserGroup;
use Admin\Entity\Delivery;
use Admin\Entity\Region;
use Admin\Entity\SelfLocation;
use Admin\Entity\SelfLocationContent;
use Admin\Service\OperLogManager;
use Doctrine\ORM\EntityManager;
use Doctrine\ORM\Query;
use Doctrine\ORM\Tools\Pagination\Paginator;
use DoctrineORMModule\Paginator\Adapter\DoctrinePaginator;
use Giggsey\Locale\Locale;
use Laminas\Config\Factory;
use Laminas\Crypt\BlockCipher;
use Laminas\Http\Client;
use Laminas\Json\Json;
use Laminas\Mvc\Controller\Plugin\AbstractPlugin;
use Laminas\Mvc\I18n\Translator;
use Laminas\Session\Container;

class AdminCommonPlugin extends AbstractPlugin
{
    private $entityManager;
    private $translator;
    private $operlogManager;
    private $sessionAdmin;

    public function __construct(
        EntityManager   $entityManager,
        Translator      $translator,
        OperLogManager  $operlogManager
    )
    {
        $this->entityManager    = $entityManager;
        $this->translator       = $translator;
        $this->operlogManager   = $operlogManager;

        if($this->sessionAdmin == null) {
            $this->sessionAdmin     = new Container('dbshopAdmin');
        }
    }

    /**
     * 管理员信息
     * @param $adminField
     * @return mixed
     */
    public function admin($adminField)
    {
        return $this->sessionAdmin->offsetGet($adminField);
    }

    /**
     * 设置语言session
     * @param $languageCode
     * @return void
     */
    public function setDbcartLanguageSession($languageCode)
    {
        $this->sessionAdmin->offsetSet('adminLanguageCode', $languageCode);
    }

    /**
     * 后台语言编码
     * @return mixed|string
     */
    public function adminLanguageCode()
    {
        $adminLanguageCode = $this->sessionAdmin->offsetGet('adminLanguageCode') ?: SHOP_ADMIN_LANGUAGE;
        //return in_array($adminLanguageCode, SHOP_LANGUAGE) ? $adminLanguageCode : SHOP_FONT_LANGUAGE;
        return $adminLanguageCode;
    }

    /**
     * 获取时区数组
     * @return mixed
     */
    public function timezoneArray()
    {
        $localLanguage = $this->translator->getTranslator()->getLocale();
        if(!file_exists('data/moduleData/System/TimezoneData/' . $localLanguage . '.php')) $localLanguage = 'zh_CN';

        return include 'data/moduleData/System/TimezoneData/' . $localLanguage . '.php';
    }

    /**
     * 获取货币数组
     * @return mixed
     */
    public function currencyArray()
    {
        $localLanguage = $this->translator->getTranslator()->getLocale();
        if(!file_exists('data/moduleData/System/CurrencyData/' . $localLanguage . '.php')) $localLanguage = 'zh_CN';
        $array = include 'data/moduleData/System/CurrencyData/' . $localLanguage . '.php';

        $currencyArray = [];
        foreach ($array['currency'] as $value) {
            $currencyArray[$value['type']] = $value['displayName'];
        }

        return $currencyArray;
    }

    /**
     * 支付列表数组，$onlinePayment 是否是在线支付，true 是，false 不是
     * @param string $onlinePayment
     * @param false $open 是否只显示已启用的在线支付
     * @return array
     */
    public function paymentList($onlinePayment = 'true', $open = false)
    {
        $paymentList = Common::paymentList($open, $onlinePayment);
        $paymentArray= [];
        foreach ($paymentList as $payValue) {
            $paymentArray[$payValue['editAction']] = $payValue['paymentName']['content'];
        }

        return $paymentArray;
    }

    /**
     * 国际电话区域数组，有地区名称的
     * @return array
     */
    public function phoneAreaArray(): array
    {
        //$languageArray  = ['zh_CN' => 'zh', 'zh_HK' => 'zh-hant-hk', 'en' => 'en', 'ja' => 'ja', 'ko' => 'ko', 'es' => 'es'];
        //$fLanguage      = Common::configValue('base', 'config')['f_language'] ?? 'zh';

        $taiwanArray        = ['zh_CN' => '台湾', 'zh_HK' => '台灣', 'en' => 'Taiwan'];
        $replaceTaiwanArray = ['zh_CN' => '中国台湾', 'zh_HK' => '中國台灣', 'en' => 'China Taiwan'];
        $phoneAreaLanguage  = Common::configValue('phoneAreaLanguage', 'language');
        $array = Locale::getAllCountriesForLocale($phoneAreaLanguage[SHOP_FONT_LANGUAGE]);
        $array['TW'] = str_replace($taiwanArray[SHOP_FONT_LANGUAGE], $replaceTaiwanArray[SHOP_FONT_LANGUAGE], $array['TW']);

        return $array;
    }

    /**
     * 验证删除的CSRF Token
     * @return bool
     */
    public function validatorCsrf(): bool
    {
        $csrfValue = $this->getController()->getRequest()->getQuery('qToken');
        if(empty($csrfValue) || $csrfValue != $this->sessionAdmin->offsetGet('queryToken')) {
            $this->getController()->flashMessenger()->addErrorMessage($this->translator->translate('不正确的请求!'));
            return false;
        }
        $this->sessionAdmin->offsetSet('queryToken', '');

        return true;
    }

    /**
     * 后台公共分页方法
     * @param Query $query
     * @param int $pageNumber
     * @param int $itemCountPerPage
     * @param bool $fetchJoinCollection
     * @return \Laminas\Paginator\Paginator
     */
    public function shopPaginator(Query $query, int $pageNumber, $itemCountPerPage = 16, $fetchJoinCollection = false)
    {
        $adapter    = new DoctrinePaginator(new Paginator($query, $fetchJoinCollection));
        $paginator  = new \Laminas\Paginator\Paginator($adapter);
        $paginator->setItemCountPerPage($itemCountPerPage);
        $paginator->setCurrentPageNumber($pageNumber);

        return $paginator;
    }

    /**
     * 返回上一页
     * @return mixed
     */
    public function toReferer()
    {
        $referer = $this->getController()->params()->fromHeader('Referer');
        if($referer) {
            $refererUrl     = $referer->uri()->getPath().(!empty($referer->uri()->getQuery()) ? '?'.$referer->uri()->getQuery() : '').(!empty($referer->uri()->getFragment()) ? '#'.$referer->uri()->getFragment() : '');
            $refererHost    = $referer->uri()->getHost();
            $host           = $this->getController()->getRequest()->getUri()->getHost();
            if ($refererUrl && $refererHost == $host) {
                return $this->getController()->redirect()->toUrl($refererUrl);
            }
        }
        return $this->getController()->redirect()->toRoute('home');
    }

    /**
     * 添加操作日志 同时输出 flashMessenger
     * @param $logBody
     * @param $operClassName
     * @param bool $message
     * @throws \Doctrine\ORM\ORMException
     * @throws \Doctrine\ORM\OptimisticLockException
     */
    public function addOperLog($logBody, $operClassName, $message = true)
    {
        $this->operlogManager->addOperLog(
            [
                'logOperUser'       => $this->sessionAdmin->offsetGet('admin_name'),
                'logOperUserGroup'  => $this->sessionAdmin->offsetGet('admin_group_name'),
                'logTime'           => time(),
                'logIp'             => $this->getController()->getRequest()->getServer('REMOTE_ADDR'),
                'logBody'           => '['.$operClassName.'] '.$logBody
            ]
        );

        if($message) $this->getController()->flashMessenger()->addSuccessMessage($logBody);
    }

    /**
     * 删除图片
     * @param $image
     * @return bool
     */
    public function deleteImage($image)
    {
        return Common::deleteImages($image);
    }

    /**
     * 获取地区下级
     * @param int $topId
     * @return array|object[]|null
     */
    public function getRegionSub($topId = 0)
    {
        $region = $this->entityManager->getRepository(Region::class)->findRegionList($topId);
        return $region ? $region : null;
    }

    /**
     * 快递信息数组
     * @param $topName
     * @param $languageCode
     * @return array|string[]
     */
    public function deliveryNameOptions($topName = '', $languageCode = '')
    {
        $deliveryList       = $this->entityManager->getRepository(Delivery::class)->findDeliveryList(['deliveryState' => 1], 'deliverySort', 'ASC', $languageCode);
        $deliveryNameArray  = [0 => empty($topName) ? $this->translator->translate('请选择') : $topName];
        if($deliveryList) foreach ($deliveryList as $deValue) {
            $deliveryNameArray[$deValue[0]->getDeliveryId()] = $deValue['deliveryName'];
        }
        return $deliveryNameArray;
    }
    /**
     * 获取动态绑定列表
     * @param string $topName
     * @return array
     */
    public function deliveryNameCodeOptions($topName = '')
    {
        $deliveryNameCodeArray   = [0 => empty($topName) ? $this->translator->translate('请选择') : $topName];
        $deliveryConfig = Common::readConfigFile('delivery');
        if (isset($deliveryConfig['deliveryApiState']) && $deliveryConfig['deliveryApiState'] == 1 && isset($deliveryConfig['deliveryApiType'])) {
            $companyArray = Common::deliveryApiCompanyCode($deliveryConfig['deliveryApiType']);
            if (!empty($companyArray)) $deliveryNameCodeArray = array_merge($deliveryNameCodeArray, $companyArray);
        }

        return $deliveryNameCodeArray;
    }

    /**
     * 获取管理员组列表
     * @param string $topName
     * @return array
     */
    public function adminGroupOptions($topName = '')
    {
        $groupList  = [0 => empty($topName) ? $this->translator->translate('选择管理组') : $topName];
        $group      = $this->entityManager->getRepository(AdminUserGroup::class)->findAdminUserGroupList($this->adminLanguageCode());
        if($group) {
            foreach ($group as $value) {
                $groupList[$value[0]->getAdminGroupId()] = $value['adminGroupName'];
            }
        }
        return $groupList;
    }

    /**
     * 自提点列表
     * @param string $topName
     * @return array|string[]
     */
    public function selfLocationOptions($topName = '')
    {
        $locationList   = [0 => empty($topName) ? $this->translator->translate('选择自提点') : $topName];
        $location       = $this->entityManager->getRepository(SelfLocationContent::class)->findBy(['languageCode' => SHOP_FONT_LANGUAGE]);
        if ($location) foreach ($location as $value) {
            $locationList[$value->getLocationId()] = $value->getLocationName();
        }
        return $locationList;
    }

    /**
     * 语言的表示图片
     * @param $language
     * @return string
     */
    public function languageImage($language): string
    {
        return '<img src="' . $this->getController()->getRequest()->getBasePath() . '/upload/common/language/'.$language.'.png' . '" class="language-image">';
    }

    /**
     * 推送服务
     * @param $action
     * @param array $data
     * @param string $path
     * @return mixed|null
     */
    public function dbshopPushService($action, $data = [], $path = '')
    {
        $serviceConfig = Common::readConfigFile('dbcartService');
        if (!$serviceConfig['key'] || !$serviceConfig['code'] || !$serviceConfig['userName']) return null;

        $blockCipher = BlockCipher::factory('openssl', ['algo' => 'aes', 'mode' => 'gcm']);
        $blockCipher->setKey($serviceConfig['key']);
        $dataStr = $blockCipher->encrypt(Json::encode(array_merge($data,
            [
                'code'          => $serviceConfig['code'],
                'scheme'        => $this->getController()->getRequest()->getUri()->getScheme(),
                'host'          => $this->getController()->getRequest()->getUri()->getHost(),
                'baseUrl'       => $this->getController()->getRequest()->getBaseUrl(),
                'port'          => $this->getController()->getRequest()->getUri()->getPort(),
                'versionNumber' => DBCART_VERSION_NUMBER,
                'phpVersion'    => PHP_VERSION
            ]
        )));

        $client = new Client(Config::SERVICE_URL . $path, [
            'adapter' => 'Laminas\Http\Client\Adapter\Curl',
            'curloptions' => [
                CURLOPT_SSL_VERIFYPEER => false
            ]
        ]);
        $client->setHeaders(['Accept' => 'application/json']);
        $client->setMethod('POST');
        $client->setParameterPost([
            'userName'      => $serviceConfig['userName'],
            'systemType'    => 'dbcart',
            'action'        => $action,
            'dataStr'       => $dataStr
        ]);

        try {
            $response = $client->send();
            if ($response->isSuccess()) {
                return Json::decode($response->getBody(), Json::TYPE_ARRAY);
            }
        } catch (\Exception $e) {
            return null;
        }

        return null;
    }

    /**
     * 服务调用
     * @param $action
     * @param array $data
     * @param string $path
     * @return mixed|null
     */
    public function dbcartApiService($action, $data = [], $path = '')
    {
        $serviceConfig = Common::readConfigFile('dbcartService');
        if (empty($path)) {
            if (!$serviceConfig['key'] || !$serviceConfig['code'] || !$serviceConfig['userName']) return null;

            $blockCipher = BlockCipher::factory('openssl', ['algo' => 'aes', 'mode' => 'gcm']);
            $blockCipher->setKey($serviceConfig['key']);
            $dataStr = $blockCipher->encrypt(Json::encode(array_merge($data,
                [
                    'code'          => $serviceConfig['code'],
                    'scheme'        => $this->getController()->getRequest()->getUri()->getScheme(),
                    'host'          => $this->getController()->getRequest()->getUri()->getHost(),
                    'baseUrl'       => $this->getController()->getRequest()->getBaseUrl(),
                    'port'          => $this->getController()->getRequest()->getUri()->getPort(),
                    'versionNumber' => DBCART_VERSION_NUMBER,
                    'phpVersion'    => PHP_VERSION
                ]
            )));
        } else {//单独调用不需要走加密接口
            $dataStr = serialize(array_merge($data,
            [
                'scheme'        => $this->getController()->getRequest()->getUri()->getScheme(),
                'host'          => $this->getController()->getRequest()->getUri()->getHost(),
                'baseUrl'       => $this->getController()->getRequest()->getBaseUrl(),
                'port'          => $this->getController()->getRequest()->getUri()->getPort(),
                'versionNumber' => DBCART_VERSION_NUMBER,
                'phpVersion'    => PHP_VERSION
            ]));
        }

        $client = new Client(Config::SERVICE_API_URL . $path, [
            'adapter' => 'Laminas\Http\Client\Adapter\Curl',
            'curloptions' => [
                CURLOPT_SSL_VERIFYPEER => false
            ]
        ]);
        $client->setHeaders(['Accept' => 'application/json']);
        $client->setMethod('POST');
        $client->setParameterPost([
            'userName'      => $serviceConfig['userName'] ?? '',
            'systemType'    => 'dbcart',
            'action'        => $action,
            'dataStr'       => $dataStr
        ]);

        try {
            $response = $client->send();
            if ($response->isSuccess()) {
                return Json::decode($response->getBody(), Json::TYPE_ARRAY);
            }
        } catch (\Exception $e) {
            return null;
        }

        return null;
    }

    /**
     * 下载服务中的包文件
     * @param $action
     * @param $data
     * @param $localPackage
     * @return bool
     */
    public function dbshopDownloadPackage($action, $data, $localPackage)
    {
        $serviceConfig  = Common::readConfigFile('dbcartService');

        $updateKey = '';
        $clientGet = new Client(Config::SERVICE_API_URL . '/getUpdateKey', ['adapter' => 'Laminas\Http\Client\Adapter\Curl', 'curloptions' => [CURLOPT_SSL_VERIFYPEER => false]]);
        $clientGet->setMethod('POST');
        $clientGet->setParameterPost(['userName' => $serviceConfig['userName'], 'actionName' => $action, 'systemType' => 'dbcart', 'hostUrl' => $this->getController()->getRequest()->getUri()->getHost()]);
        $responseGet = $clientGet->send();
        if ($responseGet->isSuccess()) $updateKey = $responseGet->getBody();
        if (empty($action) || empty($updateKey)) return false;

        $blockCipher    = BlockCipher::factory('openssl', ['algo' => 'aes', 'mode' => 'gcm']);
        $blockCipher->setKey($serviceConfig['key']);
        $dataStr = $blockCipher->encrypt(Json::encode(array_merge($data,
            [
                'code'          => $serviceConfig['code'],
                'scheme'        => $this->getController()->getRequest()->getUri()->getScheme(),
                'host'          => $this->getController()->getRequest()->getUri()->getHost(),
                'baseUrl'       => $this->getController()->getRequest()->getBaseUrl(),
                'port'          => $this->getController()->getRequest()->getUri()->getPort(),
                'updateKey'     => $updateKey,
                'versionNumber' => DBCART_VERSION_NUMBER,
                'phpVersion'    => PHP_VERSION
            ]
        )));


        $updateKeyIni = Factory::fromFile(Config::PACKAGE_UPDATE_KEY_FILE);
        $updateKeyIni[$action] = $updateKey;
        Factory::toFile(Config::PACKAGE_UPDATE_KEY_FILE, $updateKeyIni);

        $client = new Client(Config::SERVICE_API_URL, [
            'adapter' => 'Laminas\Http\Client\Adapter\Curl',
            'curloptions' => [
                CURLOPT_SSL_VERIFYPEER => false,
                CURLOPT_HEADER => 0
            ],
            'timeout' => 300
        ]);
        $client->setMethod('POST');
        $client->setParameterPost([
            'userName'      => $serviceConfig['userName'],
            'systemType'    => 'dbcart',
            'action'        => $action,
            'dataStr'       => $dataStr
        ]);
        $response = $client->send();
        if ($response->isSuccess()) {
            if ($response->getBody() == 'updateKeyError') return 'keyError';
            else {
                $signedUrl = $response->getBody();
                if (!empty($signedUrl)) {
                    $downClient     = new Client($signedUrl, ['adapter' => 'Laminas\Http\Client\Adapter\Curl', 'curloptions' => [CURLOPT_SSL_VERIFYPEER => false, CURLOPT_HEADER => 0], 'timeout' => 300]);
                    $downClient->setMethod('GET');
                    $downResponse   = $downClient->send();
                    if ($downResponse->isSuccess()) {
                        file_put_contents($localPackage, $downResponse->getBody());
                        return true;
                    }
                }
            }
        }
        return false;
    }

    /**
     * 平铺无限分类-暂时未使用
     * @param $items
     * @param string $id
     * @param string $pid
     * @param string $name
     * @param string $path
     * @param string $son
     * @return array
     */
    public function genTree(
        $items,
        $id='getGoodsCategoryId',
        $pid='getGoodsCategoryTopId',
        $name='getGoodsCategoryName',
        $path='getGoodsCategoryPath',
        $son = 'children'
    )
    {
        $tree = []; //格式化的树
        $tmpMap = [];  //临时扁平数据

        foreach ($items as $item) {
            $tmpMap[$item->$id()] = [
                'id'    => $item->$id(),
                'top_id'=> $item->$pid(),
                'name'  => $item->$name(),
                'path'  => $item->$path()
            ];
        }

        foreach ($items as $item) {
            if (isset($tmpMap[$item->$pid()])) {
                $tmpMap[$item->$pid()][$son][] = &$tmpMap[$item->$id()];
                //$tmpMap[$item->$pid()][$son][] = $tmpMap[$item->$id()];
            } else {
                $tree[] = &$tmpMap[$item->$id()];
                //$tree[] = $tmpMap[$item->$id()];
            }
        }
        unset($tmpMap);
        return $tree;
    }
}