<?php
/**
 * DBShop 电子商务系统
 *
 * ==========================================================================
 * @link      https://www.dbshop.net/
 * @copyright 北京珑大钜商科技有限公司，并保留所有权利。
 * @license   https://www.dbshop.net/license.html License
 * ==========================================================================
 *
 * @author    静静的风 <baron@loongdom.cn>
 *
 */

namespace Admin\Service\Payment;

use Admin\Data\Common;
use Laminas\Config\Writer\PhpArray;
use Stripe\StripeClient;
use Laminas\Session\Container;

class StripeService
{
    private $config         = [];
    private $intPaymentCurrency;
    private $stripe;
    private $orderSession;
    private $paymentMethodTypes;

    public function __construct()
    {
        if(!$this->config) {
            $this->config = Common::getPaymentConfig('Stripe');
        }

        $this->intPaymentCurrency = ['JPY'];

        if (!empty($this->config['privateApiKey']['content'])) $this->stripe = new StripeClient($this->config['privateApiKey']['content']);
        //$this->paymentMethodTypes = $this->config['paymentMethodTypes']['checked'] ?: ['card'];
        $this->paymentMethodTypes = ['card'];

        $this->orderSession     = new Container('stripeSession');
    }

    public function savePaymentConfig(array $data, $paymentConfig, $configFile)
    {
        $configArray = PaymentForm::setFormValue($paymentConfig, $data);

        $write = new PhpArray();
        $write->setUseBracketArraySyntax(true);
        $write->toFile($configFile, $configArray);

        Common::opcacheInvalidate($configFile);
    }

    public function getFormInput($paymentConfig)
    {
        return PaymentForm::createFormInput($paymentConfig);
    }

    /**
     * 订单支付
     * @param array $data
     * @return array
     * @throws \Stripe\Exception\ApiErrorException
     */
    public function orderPaymentTo(array $data)
    {
        $orderInfo = $data['orderInfo'];

        $html = '';
        try {
            $session = $this->stripe->checkout->sessions->create([
                'success_url' => $data['returnUrl'],
                'cancel_url' => $data['cancelUrl'],
                'payment_method_types' => $this->paymentMethodTypes,
                'line_items' => [
                    [
                        'name'      => 'Shopping OrderSn: '.$orderInfo->getOrderSn(),
                        'currency'  => $orderInfo->getCurrencyCode(),
                        'amount'    => (in_array($orderInfo->getCurrencyCode(), $this->intPaymentCurrency) ? intval($orderInfo->getOrderAmount()) : $orderInfo->getOrderAmount()) * 100,
                        'quantity'  => 1,
                    ],
                ],
                'mode' => 'payment',
                'payment_intent_data' => ['metadata' => ['orderSn' => $orderInfo->getOrderSn()]]
            ]);

            $this->orderSession->offsetSet('payment_intent', $session->payment_intent);

            $html   = '<!DOCTYPE html><html><head><title>checkout</title><link rel="stylesheet" href="style.css"><script src="https://polyfill.io/v3/polyfill.min.js?version=3.52.1&features=fetch"></script><script src="https://js.stripe.com/v3/"></script></head><body>';
            $html   .= "Go to the payment page ……<script type='text/javascript'>";
            $html   .= 'var stripe = Stripe("'.$this->config['publicApiKey']['content'].'");';
            $html   .= "stripe.redirectToCheckout({ sessionId: '".$session->id."' });";
            //$html   .= "fetch('".str_replace('shopReturn/orderPaymentReturn', 'shopCheckout/checkoutComplete', $data['returnUrl'])."', {method: 'POST',}).then(function(response) { return response.json();}).then(function(session) { return stripe.redirectToCheckout({ sessionId: session.id });}).then(function(result) {if (result.error) {alert(result.error.message);}});";
            $html   .= "</script>";
            $html   .= '</body></html>';
        } catch (\Exception $e) {
            echo $e->getMessage();
        }

        exit($html);
    }

    /**
     * 发货处理
     * @param array $data
     */
    public function orderToSend(array $data)
    {

    }

    /**
     * 支付返回
     * @param $orderInfo
     * @return array|false[]
     * @throws \Stripe\Exception\ApiErrorException
     */
    public function orderPaymentReturn($orderInfo)
    {
        $paymentIntent = $this->stripe->paymentIntents->retrieve($this->orderSession->offsetGet('payment_intent'));
        $this->orderSession->offsetUnset('payment_intent');

        if (
            $paymentIntent
            && $paymentIntent->status == 'succeeded'
            && $paymentIntent->metadata->offsetGet('orderSn') == $orderInfo->getOrderSn()
            && $paymentIntent->amount == $orderInfo->getOrderAmount() * 100
        ) {
            return ['paymentStatus' => true, 'orderAmount' => $orderInfo->getOrderAmount(), 'outTradeNo' => $paymentIntent->id];
        }
        return ['paymentStatus' => false];
    }

    /**
     * 支付异步通知(未使用，因为回调地址没有用到此方法)
     * @param $orderInfo
     * @return array
     */
    public function orderPaymentNotify($orderInfo)
    {
        return ['paymentStatus' => false, 'sendMessage' => ''];
    }

    /**
     * 支付
     * @param array $data
     */
    public function paymentTo(array $data)
    {
        $html = '';
        try {
            $session = $this->stripe->checkout->sessions->create([
                'success_url' => $data['returnUrl'],
                'cancel_url' => $data['cancelUrl'],
                'payment_method_types' => $this->paymentMethodTypes,
                'line_items' => [
                    [
                        'name'      => $data['subject'],
                        'currency'  => $data['currency'],
                        'amount'    => (in_array($data['currency'], $this->intPaymentCurrency) ? intval($data['amount']) : $data['amount']) * 100,
                        'quantity'  => 1,
                    ],
                ],
                'mode' => 'payment',
                'payment_intent_data' => ['metadata' => ['sn' => $data['sn']]]
            ]);

            $this->orderSession->offsetSet('other_payment_intent', $session->payment_intent);

            $html   = '<!DOCTYPE html><html><head><title>checkout</title><link rel="stylesheet" href="style.css"><script src="https://polyfill.io/v3/polyfill.min.js?version=3.52.1&features=fetch"></script><script src="https://js.stripe.com/v3/"></script></head><body>';
            $html   .= "Go to the payment page ……<script type='text/javascript'>";
            $html   .= 'var stripe = Stripe("'.$this->config['publicApiKey']['content'].'");';
            $html   .= "stripe.redirectToCheckout({ sessionId: '".$session->id."' });";
            //$html   .= "fetch('".str_replace('shopReturn/orderPaymentReturn', 'shopCheckout/checkoutComplete', $data['returnUrl'])."', {method: 'POST',}).then(function(response) { return response.json();}).then(function(session) { return stripe.redirectToCheckout({ sessionId: session.id });}).then(function(result) {if (result.error) {alert(result.error.message);}});";
            $html   .= "</script>";
            $html   .= '</body></html>';
        } catch (\Exception $e) {
            echo $e->getMessage();
        }

        exit($html);
    }

    /**
     * 支付返回
     * @param array $data
     * @return array|false[]
     * @throws \Stripe\Exception\ApiErrorException
     */
    public function paymentReturn(array $data)
    {
        $paymentIntent = $this->stripe->paymentIntents->retrieve($this->orderSession->offsetGet('other_payment_intent'));
        $this->orderSession->offsetUnset('other_payment_intent');

        if (
            $paymentIntent
            && $paymentIntent->status == 'succeeded'
            && $paymentIntent->metadata->offsetGet('sn') == $data['sn']
            && $paymentIntent->amount == $data['amount'] * 100
        ) {
            return ['paymentStatus' => true, 'amount' => $data['amount']];
        }
        return ['paymentStatus' => false];
    }

    public function paymentNotify(array $data)
    {

    }
}